# -*- coding: utf-8 -*-

from copy import deepcopy
from tools import trace, timer

from PyQt5.QtWidgets import (
    QMessageBox, QUndoCommand, QUndoStack,
)

from Model.Geometry import Reach


class SetDataCommand(QUndoCommand):
    def __init__(self, reach, index, old_value, new_value):
        QUndoCommand.__init__(self)

        self._reach = reach
        self._index = index
        self._old = old_value
        self._new = new_value

class SetNameCommand(SetDataCommand):
    def undo(self):
        self._reach.profile(self._index).name = self._old

    def redo(self):
        self._reach.profile(self._index).name = self._new

class SetKPCommand(SetDataCommand):
    def undo(self):
        self._reach.profile(self._index).kp = self._old

    def redo(self):
        self._reach.profile(self._index).kp = self._new


class AddCommand(QUndoCommand):
    def __init__(self, reach, index):
        QUndoCommand.__init__(self)

        self._reach = reach
        self._index = index
        self._profile = None

    def undo(self):
        self._reach.delete_profiles([self._profile])

    def redo(self):
        if self._profile is None:
            self._profile = self._reach.insert(self._index)
        else:
            self._reach.insert_profile(self._index, self._profile)

class DelCommand(QUndoCommand):
    def __init__(self, reach, rows):
        QUndoCommand.__init__(self)

        self._reach = reach
        self._rows = rows

        self._profile = []
        for row in rows:
            self._profile.append(self._reach.profile(row))
        self._profile.reverse()

    def undo(self):
        row = self._rows[0]
        for pro in self._profile:
            self._reach.insert_profile(row, pro)

    def redo(self):
        self._reach.delete(self._rows)

class SortCommand(QUndoCommand):
    def __init__(self, reach, _reverse):
        QUndoCommand.__init__(self)

        self._reach = reach
        self._reverse = _reverse

        old = self._reach.profiles
        self._reach.sort(self._reverse)
        new = self._reach.profiles

        self._indexes = list(
            map(
                lambda p: old.index(p),
                new
            )
        )

    def undo(self):
        self._reach.sort_with_indexes(self._indexes)

    def redo(self):
        self._reach.sort(self._reverse)


class MoveCommand(QUndoCommand):
    def __init__(self, reach, up, i):
        QUndoCommand.__init__(self)

        self._reach = reach
        self._up = up == "up"
        self._i = i

    def undo(self):
        if self._up:
            self._reach.move_up_profile(self._i)
        else:
            self._reach.move_down_profile(self._i)

    def redo(self):
        if self._up:
            self._reach.move_up_profile(self._i)
        else:
            self._reach.move_down_profile(self._i)


class PasteCommand(QUndoCommand):
    def __init__(self, reach, row, profiles):
        QUndoCommand.__init__(self)

        self._reach = reach
        self._row = row
        self._profiles = list(
            map(
                lambda p: deepcopy(p),
                profiles
            )
        )

    def undo(self):
        self._reach.delete_profiles(self._profiles)

    def redo(self):
        self._profiles.reverse()

        for pro in self._profiles:
            self._reach.insert_profile(self._row, pro)

        self._profiles.reverse()
